searchNodes=[{"doc":"CoderRing Provides short, unique codes upon request. Methodology The database is seeded with every possible base code. With the default of 4 characters, each being one of 32 possible characters, the database will be populated with 1,048,576 possibilities (without the expletives filter). The 32 possible characters include 2-9 and A-Z , not including I or O . This set was chosen to avoid ambiguity for human eyes. When a code is requested with get_code/2 , &quot;max&quot; is decremented by 1 from the max used in finding the last code. (If it's the first code fetched, &quot;max&quot; is the last record.) &quot;r&quot; is then a random number between 1 and &quot;max&quot;, inclusive. Records &quot;r&quot; and &quot;max&quot; are found in the table by their &quot;position&quot; field. (Records positions are numbered from 1). The values for records &quot;r&quot; and &quot;max&quot; are reversed, and the value landing out of bounds for future codes in the &quot;max&quot; position is returned as the next code. This gives off the look of truly random codes, but they never repeat. Once the cycle completes, &quot;max&quot; is reset to the last record again and it repeats. The records in the table will be randomized, but it doesn't really matter since the records will again be pulled in random order. What is a Code? A code generated by the system is comprised of three elements, concatenated together: The :extra option string supplied by the caller when invoking get_code/2 . If supplied, it should remain the same each call, or if it does change, it should not change to a previously-used value. An example use case is the year as 2 digits. For most use cases, this piece can be ignored. A &quot;uniquizer&quot; string managed by CoderRing. If the number of codes generated in the seed is not exhausted, this will be an empty string. When the ring wraps around, the uniquizer will be incremented. Also, if get_code/2 is called with the bump: true , then this will be incremented and the ring cycle reset. The base code, coming directly from the seed data in the database. Setup Include CoderRing as a dependency in your application: defp deps do [ { :coder_ring , &quot;~&gt; 0.2.0&quot; } ] end Configure code rings in your config.exs with something like: config :my_app , MyApp.CoderRing , repo : MyApp.Repo , rings : [ :widget , doodad : [ base_length : 2 ] ] Here, we configured two code rings. Note that the :rings list may have atoms for default options or keyword list-style entries ( {:doodad, base_length: 2} ) if options are specified. See CoderRing.new/1 for available options. Next, add the following to change/0 in a new or existing Ecto migration: def change do CoderRing.Migration . change ( ) end Make sure the database tables are seeded somewhere before use, whether in your existing seeds.exs or other application code: MyApp.CoderRing . populate_rings_if_empty ( ) Finally, create a CoderRing module in your application. The section below explains the options. Now you can use MyApp.CoderRing.get_code/2 to generate new codes. iex&gt; MyApp.CoderRing . get_code ( :widget ) &quot;7GRY&quot; iex&gt; MyApp.CoderRing . get_code ( :widget ) &quot;PJ83&quot; iex&gt; MyApp.CoderRing . get_code ( :widget ) &quot;NNW3&quot; iex&gt; MyApp.CoderRing . get_code ( :widget ) &quot;Q5QA&quot; Creating a CoderRing Module for your Application You'll want to create a module in your Application to expose the CoderRing functionality. Here are your options. Stateless This one is simplest, but it will require fetching current state from the database each time. defmodule MyApp.CoderRing do use CoderRing , otp_app : :my_app end GenServer-based You've chosen the BEAM for it's state-keeping prowess. In this option, a long-running GenServer is spawned which will hold the current state, avoiding the need to fetch state each time get_code/2 is called. Look out, however, if you're running with multiple servers as things will not work correctly if each node is running its own GenServer. See CoderRing.GenRing for details on setting up this method. Using a Global Process Registry If your app is running on multiple nodes with Erlang clustering enabled, another option is to spawn a GenServer under a global process registry, named by the ring name, so that only one such process is allowed to run across the cluster. For more details on this method, see Global Registry Method . Dealing with an Unexpected Duplicate Code If for some reason a code is returned which turns out not to be unique, it probably has to do with a previously-used &quot;extra&quot; string being used after switching to a different one in between. In this case, you may pass the bump: true option into get_code/2 to have the CoderRing begin using (or incrementing) the uniquizer. In this case, the ring cycle is also reset, and we should have another full cycle without conflicts. Dealing with Timeouts While I didn't have trouble loading my local Postgres in development with 1 million+ code records during initial population, I did find that when doing so on a deployment with the database over the network etc, I hit Ecto's default 15-second timeout. To solve this, you might need to increase the timeout. Try: MyApp.CoderRing . populate_rings_if_empty ( timeout : :timer . minutes ( 2 ) ) Filtering Bad Words If you wish to ensure that the generated codes do not include any profane words, also include expletive as a dependency in your application: def deps do { :expletive , &quot;~&gt; 0.1.0&quot; } end And then add the :expletive_blacklist option in the pertinent ring config: config :my_app , MyApp.CoderRing , repo : MyApp.Repo , rings : [ widget : [ expletive_blacklist : :english ] ] Acknowledgements The methodology is appreciatively derived from Robert Gamble's accepted answer on this Stack Overflow page .","ref":"CoderRing.html","title":"CoderRing","type":"behaviour"},{"doc":"Invoke the name ring with the given message .","ref":"CoderRing.html#c:call/2","title":"CoderRing.call/2","type":"callback"},{"doc":"Get the memo from db for the given ring name .","ref":"CoderRing.html#get_memo/1","title":"CoderRing.get_memo/1","type":"function"},{"doc":"Invoke the functionality identified by message . Memo should be loaded.","ref":"CoderRing.html#invoke/2","title":"CoderRing.invoke/2","type":"function"},{"doc":"Load the relevant memo from the database into ring .","ref":"CoderRing.html#load_memo/1","title":"CoderRing.load_memo/1","type":"function"},{"doc":"Make a new ring struct. Options :name - Ring name atom. Required. :base_length - Number of characters for the base code, 1-4. Default: 4 :repo - Ecto.Repo module to use. Required. :expletive_blacklist - Expletive blacklist to use: :english , :international or nil . Note that, if enabled, the expletive package must be added as a dependency in your application. Default: nil","ref":"CoderRing.html#new/1","title":"CoderRing.new/1","type":"function"},{"doc":"Load the ring into the database. All opts are passed along to Ecto.Repo calls to query and insert.","ref":"CoderRing.html#populate/2","title":"CoderRing.populate/2","type":"function"},{"doc":"Load the ring into the database if it isn't already there.","ref":"CoderRing.html#populate_if_empty/2","title":"CoderRing.populate_if_empty/2","type":"function"},{"doc":"For each ring, seed its data if it hasn't already been done. See populate/2 .","ref":"CoderRing.html#populate_rings_if_empty/2","title":"CoderRing.populate_rings_if_empty/2","type":"function"},{"doc":"Get a ring for module under otp_app by its name .","ref":"CoderRing.html#ring/3","title":"CoderRing.ring/3","type":"function"},{"doc":"List all configured rings. Memos will be unloaded. Use CoderRing.load_memo/1 to fetch state from the database.","ref":"CoderRing.html#rings/2","title":"CoderRing.rings/2","type":"function"},{"doc":":base_length - Number of characters to use in the base code. :blacklist - Set this to :english to use the Expletive package's English word blacklist. Codes with occurrences of these words will be skipped in the database seeding step. :memo - State-variable data, synced to the &quot;code_memos&quot; table. :name - Name of this coder ring: . :repo - Ecto.Repo module to use.","ref":"CoderRing.html#t:t/0","title":"CoderRing.t/0","type":"type"},{"doc":"Schema for a code, pending use.","ref":"CoderRing.Code.html","title":"CoderRing.Code","type":"module"},{"doc":"Changeset for creating a new code.","ref":"CoderRing.Code.html#changeset/2","title":"CoderRing.Code.changeset/2","type":"function"},{"doc":"Changeset for updating an existing code.","ref":"CoderRing.Code.html#update_changeset/2","title":"CoderRing.Code.update_changeset/2","type":"function"},{"doc":":id - Unique integer ID for the code. :name - Name of the code type. :position - Primary key and auto-incrementing integer. :value - A pre-generated code.","ref":"CoderRing.Code.html#t:t/0","title":"CoderRing.Code.t/0","type":"type"},{"doc":"GenServer wrapper for a CoderRing. Keeping memo state in memory with a process means some database reads can be skipped. State is, however, always synced to the database so it can be restored properly on app restart. Take care to only have one GenRing proc running for a ring at any given time. For instance, if running on a multi-server deployment, use Erlang's clustered mode and a global process registry like Horde to guarantee no more than one proc for a ring across the cluster. When running on a single server, it should be sufficient to use GenRing. Usage Create a module in your application: defmodule MyApp.CoderRing do use CoderRing.GenRing , otp_app : :my_app end Then, add it to your application supervisor: def start ( _type , _args ) do children = [ ... ] ++ MyApp.CoderRing . child_specs ( ) opts = [ ... ] { :ok , pid } = Supervisor . start_link ( children , opts ) end","ref":"CoderRing.GenRing.html","title":"CoderRing.GenRing","type":"module"},{"doc":"Schema for a coder ring state.","ref":"CoderRing.Memo.html","title":"CoderRing.Memo","type":"module"},{"doc":"Changeset for creating a new memo.","ref":"CoderRing.Memo.html#changeset/2","title":"CoderRing.Memo.changeset/2","type":"function"},{"doc":"Create a new Memo struct.","ref":"CoderRing.Memo.html#new/1","title":"CoderRing.Memo.new/1","type":"function"},{"doc":"Changeset for updating an existing memo.","ref":"CoderRing.Memo.html#update_changeset/2","title":"CoderRing.Memo.update_changeset/2","type":"function"},{"doc":":name - Name of the code type. (eg. &quot;rx&quot;) :extra - Additional prefix last used by the caller. If we get a new one, :uniquizer_num is reset to 0 and the ring is reset. :uniquizer_num - A number to be passed to integer_to_string/1 for an extra value to be used between :caller_extra and the base code. :last_max_pos - Position of the code record last used as &quot;max&quot;. Value is nil before the first iteration is made.","ref":"CoderRing.Memo.html#t:t/0","title":"CoderRing.Memo.t/0","type":"type"},{"doc":"Handles table migrations. Invoke CoderRing.Migration.change/0 in your migration's change/0 function.","ref":"CoderRing.Migration.html","title":"CoderRing.Migration","type":"module"},{"doc":"","ref":"CoderRing.Migration.html#change/0","title":"CoderRing.Migration.change/0","type":"function"},{"doc":"CoderRing Provides short, unique codes upon request.","ref":"readme.html","title":"CoderRing","type":"extras"},{"doc":"The database is seeded with every possible base code. With the default of 4 characters, each being one of 32 possible characters, the database will be populated with 1,048,576 possibilities (without the expletives filter). The 32 possible characters include 2-9 and A-Z , not including I or O . This set was chosen to avoid ambiguity for human eyes. When a code is requested with get_code/2 , &quot;max&quot; is decremented by 1 from the max used in finding the last code. (If it's the first code fetched, &quot;max&quot; is the last record.) &quot;r&quot; is then a random number between 1 and &quot;max&quot;, inclusive. Records &quot;r&quot; and &quot;max&quot; are found in the table by their &quot;position&quot; field. (Records positions are numbered from 1). The values for records &quot;r&quot; and &quot;max&quot; are reversed, and the value landing out of bounds for future codes in the &quot;max&quot; position is returned as the next code. This gives off the look of truly random codes, but they never repeat. Once the cycle completes, &quot;max&quot; is reset to the last record again and it repeats. The records in the table will be randomized, but it doesn't really matter since the records will again be pulled in random order.","ref":"readme.html#methodology","title":"CoderRing - Methodology","type":"extras"},{"doc":"A code generated by the system is comprised of three elements, concatenated together: The :extra option string supplied by the caller when invoking get_code/2 . If supplied, it should remain the same each call, or if it does change, it should not change to a previously-used value. An example use case is the year as 2 digits. For most use cases, this piece can be ignored. A &quot;uniquizer&quot; string managed by CoderRing. If the number of codes generated in the seed is not exhausted, this will be an empty string. When the ring wraps around, the uniquizer will be incremented. Also, if get_code/2 is called with the bump: true , then this will be incremented and the ring cycle reset. The base code, coming directly from the seed data in the database.","ref":"readme.html#what-is-a-code","title":"CoderRing - What is a Code?","type":"extras"},{"doc":"Include CoderRing as a dependency in your application: defp deps do [ { :coder_ring , &quot;~&gt; 0.2.0&quot; } ] end Configure code rings in your config.exs with something like: config :my_app , MyApp.CoderRing , repo : MyApp.Repo , rings : [ :widget , doodad : [ base_length : 2 ] ] Here, we configured two code rings. Note that the :rings list may have atoms for default options or keyword list-style entries ( {:doodad, base_length: 2} ) if options are specified. See CoderRing.new/1 for available options. Next, add the following to change/0 in a new or existing Ecto migration: def change do CoderRing.Migration . change ( ) end Make sure the database tables are seeded somewhere before use, whether in your existing seeds.exs or other application code: MyApp.CoderRing . populate_rings_if_empty ( ) Finally, create a CoderRing module in your application. The section below explains the options. Now you can use MyApp.CoderRing.get_code/2 to generate new codes. iex&gt; MyApp.CoderRing . get_code ( :widget ) &quot;7GRY&quot; iex&gt; MyApp.CoderRing . get_code ( :widget ) &quot;PJ83&quot; iex&gt; MyApp.CoderRing . get_code ( :widget ) &quot;NNW3&quot; iex&gt; MyApp.CoderRing . get_code ( :widget ) &quot;Q5QA&quot;","ref":"readme.html#setup","title":"CoderRing - Setup","type":"extras"},{"doc":"You'll want to create a module in your Application to expose the CoderRing functionality. Here are your options. Stateless This one is simplest, but it will require fetching current state from the database each time. defmodule MyApp.CoderRing do use CoderRing , otp_app : :my_app end GenServer-based You've chosen the BEAM for it's state-keeping prowess. In this option, a long-running GenServer is spawned which will hold the current state, avoiding the need to fetch state each time get_code/2 is called. Look out, however, if you're running with multiple servers as things will not work correctly if each node is running its own GenServer. See CoderRing.GenRing for details on setting up this method. Using a Global Process Registry If your app is running on multiple nodes with Erlang clustering enabled, another option is to spawn a GenServer under a global process registry, named by the ring name, so that only one such process is allowed to run across the cluster. For more details on this method, see Global Registry Method .","ref":"readme.html#creating-a-coderring-module-for-your-application","title":"CoderRing - Creating a CoderRing Module for your Application","type":"extras"},{"doc":"If for some reason a code is returned which turns out not to be unique, it probably has to do with a previously-used &quot;extra&quot; string being used after switching to a different one in between. In this case, you may pass the bump: true option into get_code/2 to have the CoderRing begin using (or incrementing) the uniquizer. In this case, the ring cycle is also reset, and we should have another full cycle without conflicts.","ref":"readme.html#dealing-with-an-unexpected-duplicate-code","title":"CoderRing - Dealing with an Unexpected Duplicate Code","type":"extras"},{"doc":"While I didn't have trouble loading my local Postgres in development with 1 million+ code records during initial population, I did find that when doing so on a deployment with the database over the network etc, I hit Ecto's default 15-second timeout. To solve this, you might need to increase the timeout. Try: MyApp.CoderRing . populate_rings_if_empty ( timeout : :timer . minutes ( 2 ) )","ref":"readme.html#dealing-with-timeouts","title":"CoderRing - Dealing with Timeouts","type":"extras"},{"doc":"If you wish to ensure that the generated codes do not include any profane words, also include expletive as a dependency in your application: def deps do { :expletive , &quot;~&gt; 0.1.0&quot; } end And then add the :expletive_blacklist option in the pertinent ring config: config :my_app , MyApp.CoderRing , repo : MyApp.Repo , rings : [ widget : [ expletive_blacklist : :english ] ]","ref":"readme.html#filtering-bad-words","title":"CoderRing - Filtering Bad Words","type":"extras"},{"doc":"The methodology is appreciatively derived from Robert Gamble's accepted answer on this Stack Overflow page .","ref":"readme.html#acknowledgements","title":"CoderRing - Acknowledgements","type":"extras"},{"doc":"Global Registry Method If your application is running on multiple nodes with Erlang clustering enabled, you might want the GenServer for a given ring to be running once across the cluster. (Having multiple simultaneous GenServers for the same ring will be a bad time.) Here, we'll explain one way to accomplish this. I recommend the excellent Horde library for this purpose. Follow its documentation to include it as a dependency, add its supervisor under your application supervisor, etc. Next, add a line to start the process in application.exs : def start do children = [ ... ] opts = [ strategy : :one_for_one , name : MyApp.Supervisor ] { :ok , pid } = Supervisor . start_link ( children , opts ) MyApp.CoderRing . start_rings ( ) { :ok , pid } end Finally, set up your CoderRing module to look something like this: defmodule MyApp.CoderRing do @moduledoc &quot;&quot;&quot; Wraps `CoderRing` functionality in a proc managed by Horde. Using a process to hold state means the work is quicker as state-loading database queries can be skipped. Since it runs under Horde, we are promised to have only one such instance, even though each node attempts to start the proc. &quot;&quot;&quot; use CoderRing @supervisor MyApp.MyHordeSupervisor @doc &quot;Start all configured rings.&quot; @spec start_rings :: :ok def start_rings do Enum . each ( rings ( ) , fn %{ name : name } -&gt; Horde.DynamicSupervisor . start_child ( @supervisor , child_spec ( name ) ) end ) end @doc false @spec child_spec ( any ) :: Supervisor . child_spec ( ) def child_spec ( name ) do %{ id : &quot;ring_ \#{ name } &quot; , start : { __MODULE__ , :start_link , [ name ] } } end @doc &quot;&quot;&quot; Return the `server` pid, first starting it via `child_spec`, if needed. &quot;&quot;&quot; @spec find_or_start ( GenServer . server ( ) , Supervisor . child_spec ( ) , non_neg_integer ) :: DynamicSupervisor . on_start_child ( ) def find_or_start ( name , child_spec ) do case GenServer . whereis ( name ) do nil -&gt; start_child ( child_spec ) existing_pid -&gt; { :ok , existing_pid } end end @doc &quot;&quot;&quot; Do a call to the ring with the given name. Start it if it isn&#39;t running. &quot;&quot;&quot; @spec call ( any , any ) :: any def call ( name , msg ) do with { :ok , pid } &lt;- find_or_start ( name , child_spec ( name ) ) do GenServer . call ( pid , msg ) end end @impl GenServer def init ( name ) do t = :timer . minutes ( 2 ) { :ok , name |&gt; ring ( ) |&gt; load_memo ( ) |&gt; populate_if_empty ( timeout : t ) } end @impl GenServer def handle_call ( message , _from , state ) do { reply , state } = invoke ( state , message ) { :reply , reply , state } end end Now, once your application has started, you should be able to get codes from your global process: iex&gt; MyApp.CoderRing . get_code ( :widget ) &quot;8CH4&quot; iex&gt; MyApp.CoderRing . get_code ( :widget ) &quot;GU96&quot;","ref":"global-registry-method.html","title":"Global Registry Method","type":"extras"}]